@@ -18,6 +18,9 @@ globalVariables(c("A",
                   "section",
                   "sub.section",
                   "adjusted",
+                  "symbol..",
+                  "data..",
+                  "class..",
                   "Revenue USD Mil",
                   "Shares Mil",
                   "Operating Cash Flow USD Mil",
@@ -28,4 +31,14 @@ globalVariables(c("A",
                   "Price to Sales",
                   "Book Value Per Share * USD",
                   "Cash Flow Per Share USD",
-                  "Revenue Per Share USD"))
+                  "Revenue Per Share USD",
+                  "Last.Trade.Date",
+                  "Market.Capitalization",
+                  "EBITDA",
+                  "Percent.Change.From.52.week.Low",
+                  "Percent.Change.From.50.day.Moving.Average",
+                  "Percent.Change.From.200.day.Moving.Average",
+                  "Change.in.Percent",
+                  "Ex.Dividend.Date",
+                  "Dividend.Pay.Date",
+                  "Revenue"))
