@@ -10,9 +10,10 @@
 #' @param transform_fun The transformation function from either the \code{xts},
 #' \code{quantmod}, or \code{TTR} package. Execute \code{tq_transform_fun_options()}
 #' to see the full list of options by package.
+#' @param col_rename A string or character vector containing names that can be used
+#' to quickly rename columns.
 #' @param ... Additional parameters passed to the appropriate transformation
 #' function.
-#' @param x_fun,.x,.y Deprecated. Use \code{ohlc_fun}, \code{x}, and \code{y} instead.
 #'
 #' @return Returns data in the form of a \code{tibble} object.
 #'
@@ -57,7 +58,6 @@
 #'
 #' @examples
 #' # Load libraries
-#' library(tidyverse)
 #' library(tidyquant)
 #'
 #' ##### Basic Functionality
@@ -71,7 +71,8 @@
 #'
 #' # Example 2: Use tq_transform_xy to use functions with two columns required
 #' fb_stock_prices %>%
-#'     tq_transform_xy(x = close, y = volume, transform_fun = EVWMA)
+#'     tq_transform_xy(x = close, y = volume, transform_fun = EVWMA,
+#'                     col_rename = "EVMWA")
 #'
 #' # Example 3: Using tq_transform_xy to work with non-OHLC data
 #' tq_get("DCOILWTICO", get = "economic.data") %>%
@@ -87,34 +88,125 @@
 
 # PRIMARY FUNCTIONS ----
 
-tq_transform <- function(data, ohlc_fun = OHLCV, transform_fun, x_fun, ...) {
-
-    # Deprecation
-    if (!missing(x_fun)) {
-        warning("argument x_fun is deprecated; please use ohlc_fun instead.",
-                call. = FALSE)
-        ohlc_fun <- deparse(substitute(x_fun))
-    } else {
-        ohlc_fun <- deparse(substitute(ohlc_fun))
-    }
+#' @rdname tq_transform
+#' @export
+tq_transform <- function(data, ohlc_fun = OHLCV, transform_fun, col_rename = NULL, ...) {
 
     # Convert to NSE
+    ohlc_fun <- deparse(substitute(ohlc_fun))
     transform_fun <- deparse(substitute(transform_fun))
 
-    tq_transform_(data, ohlc_fun, transform_fun, ...)
+    # Patch for grouped data frames
+    if (dplyr::is.grouped_df(data)) {
+
+        tq_transform_grouped_df_(data, ohlc_fun, transform_fun,
+                                 col_rename = col_rename, ...)
+
+    } else {
+
+        tq_transform_base_(data, ohlc_fun, transform_fun,
+                           col_rename = col_rename, ...)
+
+    }
+}
+
+#' @rdname tq_transform
+#' @export
+tq_transform_ <- function(data, ohlc_fun = "OHLCV", transform_fun, col_rename = NULL, ...) {
+
+    # Patch for grouped data frames
+    if (dplyr::is.grouped_df(data)) {
+
+        tq_transform_grouped_df_(data, ohlc_fun, transform_fun,
+                                 col_rename = col_rename, ...)
+
+    } else {
+
+        tq_transform_base_(data, ohlc_fun, transform_fun,
+                           col_rename = col_rename, ...)
+
+    }
 
 }
 
 #' @rdname tq_transform
 #' @export
-tq_transform_ <- function(data, ohlc_fun = "OHLCV", transform_fun, x_fun, ...) {
+tq_transform_xy <- function(data, x, y = NULL, transform_fun, col_rename = NULL, ...) {
+
+    # Convert to NSE
+    x <- deparse(substitute(x))
+    y <- deparse(substitute(y))
+    transform_fun <- deparse(substitute(transform_fun))
+
+    # Patch for grouped data frames
+    if (dplyr::is.grouped_df(data)) {
+
+        tq_transform_xy_grouped_df_(data, x, y, transform_fun,
+                                    col_rename = col_rename, ...)
+
+    } else {
+
+        tq_transform_xy_base_(data, x, y, transform_fun,
+                              col_rename = col_rename, ...)
 
-    # Deprecation
-    if (!missing(x_fun)) {
-        warning("argument x_fun is deprecated; please use ohlc_fun instead.",
-                call. = FALSE)
-        ohlc_fun <- x_fun
     }
+}
+
+#' @rdname tq_transform
+#' @export
+tq_transform_xy_ <- function(data, x, y = NULL, transform_fun, col_rename = NULL, ...) {
+
+    # Patch for grouped data frames
+    if (dplyr::is.grouped_df(data)) {
+
+        tq_transform_xy_grouped_df_(data, x, y, transform_fun,
+                                    col_rename = col_rename, ...)
+
+    } else {
+
+        tq_transform_xy_base_(data, x, y, transform_fun,
+                              col_rename = col_rename, ...)
+
+    }
+}
+
+#' @rdname tq_transform
+#' @export
+tq_transform_fun_options <- function() {
+
+    # zoo rollapply functions
+    pkg_regex_zoo <- "roll"
+    funs_zoo <- ls("package:zoo")[stringr::str_detect(ls("package:zoo"), pkg_regex_zoo)]
+
+    # xts apply.period, to.period, lag and diff functions
+    pkg_regex_xts <- "apply|to\\.|period|lag|diff"
+    funs_xts <- ls("package:xts")[stringr::str_detect(ls("package:xts"), pkg_regex_xts)]
+
+    # quantmod periodReturns, Delt, series functions
+    pkg_regex_quantmod <- "Return|Delt|Lag|Next|^Op..|^Cl..|^Hi..|^Lo..|^series"
+    funs_quantmod <- ls("package:quantmod")[stringr::str_detect(ls("package:quantmod"), pkg_regex_quantmod)]
+
+    # TTR functions
+    pkg_regex_ttr <- "^get*|^stock|^naCh" # NOT these
+    funs_ttr <- ls("package:TTR")[!stringr::str_detect(ls("package:TTR"), pkg_regex_ttr)]
+
+    fun_options <- list(zoo = funs_zoo,
+                        xts = funs_xts,
+                        quantmod = funs_quantmod,
+                        TTR = funs_ttr)
+
+    fun_options
+
+}
+
+
+# UTILITY FUNCTIONS ----
+
+# See utils-date.R for date and datetime functions
+
+# Base functions ----
+
+tq_transform_base_ <- function(data, ohlc_fun, transform_fun, col_rename, ...) {
 
     # Check transform_fun in xts, quantmod or TTR
     check_transform_fun_options(transform_fun)
@@ -131,6 +223,12 @@ tq_transform_ <- function(data, ohlc_fun = "OHLCV", transform_fun, x_fun, ...) {
     # Get timezone
     time_zone <- get_time_zone(data, date_col_name)
 
+    # Drop any non-numeric columns except for date
+    date_col <- dplyr::select_(data, date_col_name)
+    numeric_cols <- data %>%
+        dplyr::select_if(is.numeric)
+    data <- dplyr::bind_cols(date_col, numeric_cols)
+
     # Convert inputs to functions
     ohlc_fun <- paste0("quantmod::", ohlc_fun)
     fun_x <- eval(parse(text = ohlc_fun))
@@ -155,56 +253,14 @@ tq_transform_ <- function(data, ohlc_fun = "OHLCV", transform_fun, x_fun, ...) {
     }
 
     # Coerce to tibble and convert date / datetime
-    if (xts::is.xts(ret)) ret <- coerce_to_tibble(ret, date_col_name, time_zone)
+    if (xts::is.xts(ret)) ret <- coerce_to_tibble(ret, date_col_name,
+                                                  time_zone, col_rename)
 
     ret
 
 }
 
-#' @rdname tq_transform
-#' @export
-tq_transform_xy <- function(data, x, y = NULL, transform_fun, .x, .y, ...) {
-
-    # Deprecation
-    if (!missing(.x)) {
-        warning("argument .x is deprecated; please use x instead.",
-                call. = FALSE)
-        x <- deparse(substitute(.x))
-    } else {
-        x <- deparse(substitute(x))
-    }
-
-    if (!missing(.y)) {
-        warning("argument .y is deprecated; please use y instead.",
-                call. = FALSE)
-        y <- deparse(substitute(.y))
-    } else {
-        y <- deparse(substitute(y))
-    }
-
-    # Convert to NSE
-    transform_fun <- deparse(substitute(transform_fun))
-
-    tq_transform_xy_(data, x, y, transform_fun, ...)
-
-}
-
-#' @rdname tq_transform
-#' @export
-tq_transform_xy_ <- function(data, x, y = NULL, transform_fun, .x, .y, ...) {
-
-    # Deprecation
-    if (!missing(.x)) {
-        warning("argument .x is deprecated; please use x instead.",
-                call. = FALSE)
-        x <- .x
-    }
-
-    if (!missing(.y)) {
-        warning("argument .y is deprecated; please use y instead.",
-                call. = FALSE)
-        y <- .y
-    }
+tq_transform_xy_base_ <- function(data, x, y, transform_fun, col_rename, ...) {
 
     # Check transform_fun in xts, quantmod or TTR
     check_transform_fun_options(transform_fun)
@@ -221,6 +277,12 @@ tq_transform_xy_ <- function(data, x, y = NULL, transform_fun, .x, .y, ...) {
     # Get timezone
     time_zone <- get_time_zone(data, date_col_name)
 
+    # Drop any non-numeric columns except for date
+    date_col <- dplyr::select_(data, date_col_name)
+    numeric_cols <- data %>%
+        dplyr::select_if(is.numeric)
+    data <- dplyr::bind_cols(date_col, numeric_cols)
+
     # Convert inputs to functions
     fun_transform <- eval(parse(text = transform_fun))
 
@@ -261,45 +323,53 @@ tq_transform_xy_ <- function(data, x, y = NULL, transform_fun, .x, .y, ...) {
     }
 
     # Coerce to tibble and convert date / datetime
-    if (xts::is.xts(ret)) ret <- coerce_to_tibble(ret, date_col_name, time_zone)
+    if (xts::is.xts(ret)) ret <- coerce_to_tibble(ret, date_col_name,
+                                                  time_zone, col_rename)
 
     ret
 
 }
 
-#' @rdname tq_transform
-#' @export
-tq_transform_fun_options <- function() {
+# Patches for grouped data frames -----
 
-    # zoo rollapply functions
-    pkg_regex_zoo <- "roll"
-    funs_zoo <- ls("package:zoo")[stringr::str_detect(ls("package:zoo"), pkg_regex_zoo)]
+tq_transform_grouped_df_ <- function(data, ohlc_fun, transform_fun, col_rename, ...) {
 
-    # xts apply.period, to.period, lag and diff functions
-    pkg_regex_xts <- "apply|to\\.|period|lag|diff"
-    funs_xts <- ls("package:xts")[stringr::str_detect(ls("package:xts"), pkg_regex_xts)]
-
-    # quantmod periodReturns, Delt, series functions
-    pkg_regex_quantmod <- "Return|Delt|Lag|Next|^Op..|^Cl..|^Hi..|^Lo..|^series"
-    funs_quantmod <- ls("package:quantmod")[stringr::str_detect(ls("package:quantmod"), pkg_regex_quantmod)]
-
-    # TTR functions
-    pkg_regex_ttr <- "^get*|^stock|^naCh" # NOT these
-    funs_ttr <- ls("package:TTR")[!stringr::str_detect(ls("package:TTR"), pkg_regex_ttr)]
-
-    fun_options <- list(zoo = funs_zoo,
-                        xts = funs_xts,
-                        quantmod = funs_quantmod,
-                        TTR = funs_ttr)
-
-    fun_options
+    group_names <- dplyr::groups(data)
 
+    data %>%
+        tidyr::nest() %>%
+        dplyr::mutate(nested.col = data %>%
+                          purrr::map(~ tq_transform_base_(data = .x,
+                                                          ohlc_fun = ohlc_fun,
+                                                          transform_fun = transform_fun,
+                                                          col_rename = col_rename,
+                                                          ...))
+        ) %>%
+        dplyr::select(-data) %>%
+        tidyr::unnest() %>%
+        dplyr::group_by_(.dots = group_names)
 }
 
+tq_transform_xy_grouped_df_ <- function(data, x, y, transform_fun, col_rename, ...) {
 
-# UTILITY FUNCTIONS ----
+    group_names <- dplyr::groups(data)
 
-# See utils.R for find_date_cols
+    data %>%
+        tidyr::nest() %>%
+        dplyr::mutate(nested.col = data %>%
+                          purrr::map(~ tq_transform_xy_base_(data = .x,
+                                                             x = x,
+                                                             y = y,
+                                                             transform_fun = transform_fun,
+                                                             col_rename = col_rename,
+                                                             ...))
+        ) %>%
+        dplyr::select(-data) %>%
+        tidyr::unnest() %>%
+        dplyr::group_by_(.dots = group_names)
+}
+
+# Checks -----
 
 check_transform_fun_options <- function(fun) {
     fun_options <- tq_transform_fun_options() %>%
@@ -330,7 +400,9 @@ check_x_y_valid <- function(data, x, y) {
     }
 }
 
-coerce_to_tibble <- function(data, date_col_name, time_zone, transform_fun) {
+# Other -----
+
+coerce_to_tibble <- function(data, date_col_name, time_zone, col_rename) {
 
     # Coerce to tibble
     ret <- data %>%
@@ -343,6 +415,15 @@ coerce_to_tibble <- function(data, date_col_name, time_zone, transform_fun) {
     # Rename row.names
     names(ret)[[1]] <- date_col_name
 
+    # Rename columns
+    if (!is.null(col_rename)) {
+        if (length(col_rename) == length(names(ret)) - 1) {
+            names(ret)[2:length(names(ret))] <- col_rename
+        } else {
+            warning("Could not rename columns")
+        }
+    }
+
     ret
 }
 
@@ -355,4 +436,3 @@ detect_period_fun <- function(fun) {
     is_period_fun
 }
 
-
